// @author gerry.gold@gmail.com
//
// MIT License

var TotalRecall = function() {

    // By default there is no timeout
    var _timeout = null;

    var _lootBag = {};

    var _getNow = function() {
        return parseInt( Date.now() / 1000, 10 );
    };

    var _isCacheStale = function( timestamp ) {
        return ( _getNow() - timestamp ) > _timeout;
    };

    this.setTimeoutInSeconds = function( timeInSeconds ) {
        _timeout = timeInSeconds;
    };

    this.put = function( key, val ) {
        _lootBag[key] = {v: val, t: _getNow()};
    };

    // If key does not exist:
    //   if 2nd arg is passed, i.e., default is defined, return default
    //   if 2nd arg is not passed, return null
    this.get = function( key, default_value ) {
        if ( key in _lootBag ) {
            if ( _timeout === null ) {
                return _lootBag[key].v;
            } else {
                if ( _isCacheStale(_lootBag[key].t) ) {
                    this.removeKey( key );
                    return ( typeof(default_value) === "undefined" ) ? null : default_value;
                } else {
                    return _lootBag[key].v;
                }
            }
        } else {
            return ( typeof(default_value) === "undefined" ) ? null : default_value;
        }
    };

    // If a TTL is set and the key has expired, its existence is set free. :)
    this.keyExists = function( key ) {
        if ( key in _lootBag ) {
            if ( _timeout === null ) {
                return true;
            } else {
                if ( _isCacheStale(_lootBag[key].t) ) {
                    this.removeKey( key );
                    return false;
                } else {
                    return true;
                }
            }
        } else {
            return false;
        }
    };

    this.removeKey = function( key ) {
        if ( key in _lootBag ) {
            delete _lootBag[key];
        }
    };

    this.clear = function() {
        _lootBag = {};    
    };

    this.keys = function() {
        return Object.keys(_lootBag);
    };
};

module.exports = TotalRecall;
